package com.util;

import java.io.File;
import java.io.IOException;
import java.util.List;
import net.jimmc.jshortcut.JShellLink;
import org.eclipse.swt.program.Program;
import com.model.DosboxVersion;
import com.model.Profile;
import com.model.Settings;


public final class PlatformUtils {

	public static final String OS_NAME = System.getProperty("os.name");
	public static final boolean IS_WINDOWS = OS_NAME.startsWith("Windows");
	public static final boolean IS_LINUX = OS_NAME.startsWith("Linux");
	public static final boolean IS_OSX = OS_NAME.startsWith("Mac OS");
	public static final String DB_EXECUTABLE = IS_WINDOWS? "DOSBox.exe": "dosbox";
	public static final String EOLN = System.getProperty("line.separator");
	
	
	public static String toNativePath(final String dbFileLocation) {
	    return dbFileLocation.replace('\\', File.separatorChar);
	}

	public static String toDosboxPath(final String hostFileLocation) {
	    return hostFileLocation.replace(File.separatorChar, '\\');
	}
	
	public static String archiveToNativePath(final String archiveEntry) {
	    // 7zip uses / as file seperator but to be safe in the future
	    // both \\ and / are converted to the host seperator
	    return archiveEntry.replace('/', File.separatorChar).replace('\\', File.separatorChar);
	}

	public static void openForEditing(final File file) {
	    Program prog = Program.findProgram(".conf");
	    if (prog == null) { prog = Program.findProgram(".txt");}
	    if (prog == null) { prog = Program.findProgram(".inf");}
	    if (prog == null) { prog = Program.findProgram(".ini");}
	    if (prog == null) { prog = Program.findProgram(".cfg");}
	    if (prog == null) { prog = Program.findProgram(".java");}
	    if (prog == null) { prog = Program.findProgram(".doc");}
	    if (prog == null) {
	        System.err.println(Settings.getInstance().msg("general.error.openfile", new Object[] {file}));
	    } else {
	        prog.execute(file.getPath());
	    }
	}
	
	public static void openForBrowsing(final String url) {
	    if (IS_WINDOWS) {
	        try {
	            Runtime.getRuntime().exec("rundll32 url.dll,FileProtocolHandler " + url);
	        } catch (IOException e) {
	            System.err.println(Settings.getInstance().msg("general.error.openurl", new Object[] {url}));
	        } 
	    } else {
	        Program prog = Program.findProgram(".html");
	        if (prog == null) {
	            System.err.println(Settings.getInstance().msg("general.error.openurl", new Object[] {url}));
	        } else {
	            prog.execute (url);
	        }
	    }
	}
	
	public static void openDirForViewing(final File file) {
	    try {
	        if (IS_WINDOWS) {
	            Runtime.getRuntime().exec(new String[]{"explorer", file.getPath()}, null, null);
	        } else if (IS_LINUX) {
	            try {
	                Runtime.getRuntime().exec(new String[]{"nautilus", file.getPath()}, null, null);
	            } catch (IOException e) {
	                Runtime.getRuntime().exec(new String[]{"kfmclient", file.getPath()}, null, null);
	            }
	        } else if (IS_OSX) {
	            Runtime.getRuntime().exec(new String[]{"open", file.getPath()}, null, null);
	        }
	    } catch (IOException e) {
	        System.err.println(Settings.getInstance().msg("general.error.opendir", new Object[] {file}));
	    }
	}

	public static void createShortcut(final Profile profile, final List<DosboxVersion> dbversionsList) {
	    int dbversionIndex = DosboxVersion.findById(dbversionsList, profile.getDbversionId());
	    DosboxVersion dbversion = dbversionsList.get(dbversionIndex);
	    StringBuffer params = new StringBuffer(128);
	    if (dbversion.isMultiConfig()) {
	        params.append("-conf \"").append(dbversion.getCanonicalConfFile()).append("\" ");
	    }
	    params.append("-conf \"").append(profile.getCanonicalConfFile()).append('"');
	    if (Settings.getInstance().getBooleanValue("dosbox", "hideconsole")) {
	        params.append(" -noconsole");
	    }
	    JShellLink link = new JShellLink();
	    link.setFolder(JShellLink.getDirectory("desktop"));
	    link.setName(profile.getTitle().replaceAll("[:?*]", " "));
	    link.setPath(dbversion.getCanonicalExecutable().getPath());
	    link.setArguments(params.toString());
	    link.setDescription(Settings.getInstance().msg("general.shortcut.title", new Object[] {profile.getTitle()}));
	    link.setWorkingDirectory(FileUtils.getDosRoot());
	    link.save();
	}
}
